from unittest.mock import call

from dell_ai.system_utils.base import Printer
from dell_ai.system_utils.k8s_info import K8SInfo, get_kube_info


def test_get_kube_info(commandline_patches):
    """
    Test standard k8s_info population
    """
    k8s_info = get_kube_info()
    assert k8s_info.server_platform == "linux/amd64"
    assert k8s_info.server_version == "v1.32.9"
    assert k8s_info.node_kubelet_version == ["v1.32.9"]


def test_kubectl_not_found(fp):
    """
    Test output when kubectl is not installed
    """
    fp.register(
        ["kubectl", fp.any()],
        stdout="kubectl: command not found",
        returncode=1,
        occurrences=2,
    )
    kubeinfo = get_kube_info()
    assert kubeinfo == K8SInfo(
        server_version=None, server_platform=None, node_kubelet_version=[]
    )


def test_k8s_info_compare_success(printer_echo_mock):
    """
    Test comparison success case, when version and platform are already tested. 
    Should not generated any output
    """
    success = K8SInfo(
        server_version="v1.29.3",
        server_platform="linux/amd64",
        node_kubelet_version=["v1.29.3", "v1.28.1"],
    )
    others = [
        K8SInfo(
            server_version="v1.29.3",
            server_platform="linux/amd64",
            node_kubelet_version=["v1.29.3"],
        ),
        K8SInfo(
            server_version="v1.28.1",
            server_platform="linux/arm64",
            node_kubelet_version=["v1.28.1", "v1.29.3"],
        ),
    ]

    success.compare(others)

    printer_echo_mock.assert_not_called()


def test_k8s_info_compare_failure(printer_echo_mock):
    """
    Test comparison failures, when version and platforms are different from the ones tested
    """
    # server_version is higher, should warn
    # server_platform not in tested, should generate error
    # node_kubelet_version is lower, should generate error
    failure = K8SInfo(
        server_version="v1.30.0",
        server_platform="linux/ppc64le",
        node_kubelet_version=["v1.27.5"],
    )

    others = [
        K8SInfo(
            server_version="v1.28.1",
            server_platform="linux/amd64",
            node_kubelet_version=["v1.28.1"],
        ),
        K8SInfo(
            server_version="v1.29.3",
            server_platform="linux/arm64",
            node_kubelet_version=["v1.29.3"],
        ),
    ]

    failure.compare(others)

    expected_simple_compare_calls = [
        call(
            Printer.version_compare_styled(
                tag="Kubernetes Server Version",
                attr_name="server_version",
                self_value="v1.30.0",
                min_supported_value="1.28.1",
                max_supported_value="1.29.3",
            ),
            level="warn",
        ),
        call(
            Printer.list_compare_styled(
                tag="Kubernetes Platform Version",
                attr_name="server_platform",
                self_value="linux/ppc64le",
                supported_values=["linux/amd64", "linux/arm64"],
            ),
            level="error",
        ),
        call(
            f"Node Kubelet version v1.27.5 is lower than minimum supported kubelet version 1.28.1",
            level="error",
        ),
    ]

    assert printer_echo_mock.call_count == 3

    printer_echo_mock.assert_has_calls(expected_simple_compare_calls, any_order=True)



import platform
from typing import List, Optional

import pytest
from pydantic_extra_types.semantic_version import SemanticVersion
from typing_extensions import Self

from dell_ai.system_utils.base import ComparableBaseModel, Printer, cmd_stdout


def test_cmd_stdout():
    """
    Test the functionality of the cmd_stdout function directly on executor
    """
    if platform.system() == "Linux":
        assert cmd_stdout(["ls"])
        assert cmd_stdout(["xyz"]) is None


def test_cmd_stdout_patched(fp):
    """
    Test the patched function cmd_stdout with different stdout values and return codes.
    """
    fp.register(["xyz"], stdout="some output")
    assert cmd_stdout(["xyz"]) == "some output"
    fp.register(["xyz"], stdout="xyz: command not found", returncode=1)
    assert cmd_stdout(["xyz"]) is None


def test_simple_list_compare(monkeypatch, printer_echo_mock):
    """
    Test the functionality of the simple_list_compare function
    
    If Val is not in the list of other values, we should obtain an output pointing this out, otherwise no output is generated.
    """
    class Obj(ComparableBaseModel):
        val: int | None = None

        def compare(self, others: List[Self]):
            self.simple_list_compare("val", others, "Val")

    success = Obj(val=1)
    failure = Obj(val=5)
    others = [Obj(val=1), Obj(val=2)]

    success.compare(others)
    printer_echo_mock.assert_not_called()

    none_obj = Obj()
    none_obj.compare(others)
    printer_echo_mock.assert_called_with(Printer.not_found("Val", "val"), level="error")

    failure.compare(others)
    printer_echo_mock.assert_called_with(
        Printer.list_compare_styled(
            tag="Val", attr_name="val", supported_values=[1, 2], self_value=5
        ),
        level="info",
    )


def test_simple_list_compare_attr_check(monkeypatch):
    """
    Test the functionality of the simple_list_compare function with wrong attribute check, error should be generated if wrong attribute is passed for comparison.
    """
    class Obj(ComparableBaseModel):
        val: Optional[int] = None

        def compare(self, others: List[Self]):
            self.simple_list_compare(
                "val1", others, "Val"
            )  # wrong attribute provided, should raise AttributeError

    success = Obj(val=1)
    others = [Obj(val=1), Obj(val=2)]
    with pytest.raises(AttributeError):
        success.compare(others)


def test_more_than_at_least_one(monkeypatch, printer_echo_mock):
    """
    Test values compares, if values are less than at least one in the list, we should get an error.
    If values are more, then a warning is generated. Otherwise no output is generated.
    """
    class Obj(ComparableBaseModel):
        val: int | None = None

        def compare(self, others: List[Self]):
            self.more_than_at_least_one("val", others, "Val")

    success = Obj(val=5)
    failure = Obj(val=1)
    others = [Obj(val=2), Obj(val=3)]

    success.compare(others)
    printer_echo_mock.assert_not_called()

    none_obj = Obj()
    none_obj.compare(others)
    printer_echo_mock.assert_called_with(Printer.not_found("Val", "val"), level="error")

    failure.compare(others)
    printer_echo_mock.assert_called_with(
        Printer.minimum_styled(
            tag="Val", self_value=1, supported_values=[2, 3], attr_name="val"
        ),
        level="info",
    )


def test_version_compare_simple_cases(monkeypatch, printer_echo_mock):
    """
    Similar to above but comparing versions
    """
    class Obj(ComparableBaseModel):
        v: str | int | None = None

        def compare(self, others: List[Self]):
            self.software_version_compare("v", others, "Val")

    success = Obj(v="1.2.3")
    error = Obj(v="0.0.1")
    warn = Obj(v="2.0.0")
    others = [Obj(v="0.1.1"), Obj(v="1.2.3"), Obj(v="1.5.1")]
    others_failing_first = [Obj(v="1.2")]
    others_failing_second = [Obj(v=1)]
    
    # version in the list
    success.compare(others)
    printer_echo_mock.assert_not_called()

    # no version, None
    none_obj = Obj()
    none_obj.compare(others)
    printer_echo_mock.assert_called_with(Printer.not_found("Val", "v"), level="error")

    # Version lower than all in list
    error.compare(others)
    printer_echo_mock.assert_called_with(
        Printer.version_compare_styled(
            tag="Val",
            attr_name="v",
            self_value="0.0.1",
            min_supported_value="0.1.1",
            max_supported_value="1.5.1",
            greater=False,
        ),
        level="error",
    )

    # version higher than all in list
    warn.compare(others)
    printer_echo_mock.assert_called_with(
        Printer.version_compare_styled(
            tag="Val",
            attr_name="v",
            self_value="2.0.0",
            min_supported_value="0.1.1",
            max_supported_value="1.5.1",
            greater=True,
        ),
        level="warn",
    )

    # incorrect semantic version format
    success.compare(others_failing_first)
    printer_echo_mock.assert_called_with(
        "Comparison Val (v) 1.2 cannot be parsed as semantic version", level="error"
    )

    # version check with anything other than a string input
    success.compare(others_failing_second)
    printer_echo_mock.assert_called_with(
        "Comparison Val (v) 1 is not a string", level="error"
    )

    # other way round for above
    Obj(v="1.2").compare(others)
    printer_echo_mock.assert_called_with(
        "This Val (v) 1.2 cannot be parsed as semantic version", level="error"
    )

    Obj(v=1).compare(others)
    printer_echo_mock.assert_called_with(
        "This Val (v) 1 is not a string", level="error"
    )


def test_version_semver():
    """
    Basic semantic version validation
    """
    assert SemanticVersion.parse("1.2.3")
    with pytest.raises(ValueError):
        SemanticVersion.parse("v1.2.3")

    assert SemanticVersion.parse("1.2.3") == SemanticVersion.parse("1.2.3")
    assert SemanticVersion.parse("1.2.3") < SemanticVersion.parse("2.0.0")

    assert sorted([SemanticVersion.parse("2.0.0"), SemanticVersion.parse("1.2.3")]) == [
        SemanticVersion.parse("1.2.3"),
        SemanticVersion.parse("2.0.0"),
    ]


def test__version_parse():
    """
    Test version parser
    """
    parser = ComparableBaseModel._version_parse
    assert parser("1.2.3") == SemanticVersion.parse("1.2.3")
    assert parser("1.2.03") == SemanticVersion.parse("1.2.3")
    assert parser("v1.2.3") == SemanticVersion.parse("1.2.3")
    assert parser("v1.2.03") == SemanticVersion.parse("1.2.3")



"""
Common test fixtures for the dell-ai package.
"""

import json
import platform
from pathlib import Path
from unittest.mock import Mock, patch

import pytest

from dell_ai.client import DellAIClient
from dell_ai.system_utils import mem_info, os_info
from dell_ai.system_utils.base import Printer


@pytest.fixture
def mock_api_response():
    """Fixture that returns a mock API response."""
    return {
        "models": [
            {
                "repoName": "test-org/test-model",
                "description": "Test model",
                "license": "apache-2.0",
                "creatorType": "user",
                "size": 1000000,
                "hasSystemPrompt": False,
                "isMultimodal": False,
                "status": "active",
            }
        ],
        "platforms": [
            {
                "id": "test-sku",
                "name": "Test Platform",
                "disabled": False,
                "server": "test-server",
                "vendor": "test-vendor",
                "gputype": "test-gpu",
                "gpuram": "16GB",
                "gpuinterconnect": "test-interconnect",
                "productName": "Test Product",
                "totalgpucount": 4,
                "interconnect-east-west": "test-east-west",
                "interconnect-north-south": "test-north-south",
            }
        ],
    }


@pytest.fixture
def mock_client(mock_api_response):
    """Fixture that returns a mock DellAIClient instance."""
    with patch("dell_ai.client.requests") as mock_requests:
        mock_response = Mock()
        mock_response.json.return_value = mock_api_response
        mock_requests.get.return_value = mock_response

        client = DellAIClient(token="test-token")
        yield client


@pytest.fixture
def patched_platform(monkeypatch, fp):
    """
    Fixture to mock the uname result, required for the platform information
    """
    resource_path = Path(__file__).parent / "unit" / "system_info" / "resources"
    class uname_result:
        system = "Linux"
        node = "deh-r760xaL40-53"
        release = "6.8.0-88-generic"
        version = "89-Ubuntu SMP PREEMPT_DYNAMIC Sat Oct 11 01:02:46 UTC 2025"
        machine = "x86_64"

    monkeypatch.setattr(platform, "uname", lambda: uname_result())
    monkeypatch.setattr(
        platform,
        "freedesktop_os_release",
        lambda: {
            "NAME": "Ubuntu",
            "ID": "ubuntu",
            "PRETTY_NAME": "Ubuntu 24.04.3 LTS",
            "VERSION_ID": "24.04",
            "VERSION": "24.04.3 LTS (Noble Numbat)",
            "VERSION_CODENAME": "noble",
            "ID_LIKE": "debian",
            "HOME_URL": "https://www.ubuntu.com/",
            "SUPPORT_URL": "https://help.ubuntu.com/",
            "BUG_REPORT_URL": "https://bugs.launchpad.net/ubuntu/",
            "PRIVACY_POLICY_URL": "https://www.ubuntu.com/legal/terms-and-policies/privacy-policy",
            "UBUNTU_CODENAME": "noble",
            "LOGO": "ubuntu-logo",
        },
    )
    fp.register(
        ["hostnamectl", "--json", "short"],
        stdout=json.dumps({"HardwareModel": "PowerEdge R760xa"})
    )
    fp.register(["dmidecode", "-s", "system-product-name"], stdout="PowerEdge R760xa")
    monkeypatch.setattr(os_info, "DMI_FILE_PATH", resource_path / "dmi_file.txt")


@pytest.fixture
def commandline_patches(fp, monkeypatch, patched_platform):
    """
    Fixture that patches all subprocess CLI commands used by the system_info module. 
    """
    resource_path = Path(__file__).parent / "unit" / "system_info" / "resources"

    fp.register(
        ["lscpu", "--json"], stdout=(resource_path / "lscpu_response.json").read_text()
    )
    fp.register(
        ["lspci", "-nn"],
        stdout=(resource_path / "lspci_stdout_nvidia_gpu.txt").read_text(),
        occurrences=2,
    )
    fp.register(
        [
            "nvidia-smi",
            "--query-gpu=name,driver_version,memory.total,compute_cap",
            "--format=csv,noheader",
        ],
        stdout=(resource_path / "nvidia_gpu_info.txt").read_text(),
    )
    fp.register(
        ["nvidia-smi"],
        stdout=(resource_path / "nvidia_smi.txt").read_text(),
        occurrences=2,
    )
    fp.register(
        ["nvidia-ctk", "--version"],
        stdout=(resource_path / "nvidia_ctk.txt").read_text(),
        occurrences=2,
    )
    fp.register(
        ["/usr/bin/nvidia-container-runtime-hook", "--version"],
        stdout=(resource_path / "nvidia_container_runtime.txt").read_text(),
        occurrences=2,
    )
    fp.register(
        ["kubectl", "get", "nodes", "-o", "json"],
        stdout=(resource_path / "kubectl_get_nodes.json").read_text(),
        occurrences=4,
    )
    fp.register(
        ["kubectl", "version", "-o", "json"],
        stdout=(resource_path / "kubectl_version.json").read_text(),
    )
    fp.register(
        ["lsblk", "-o", "name,model,size,type,mountpoint", "--json"],
        stdout=(resource_path / "lsblk.json").read_text(),
    )

    monkeypatch.setattr(mem_info, "MEMINFO_PATH", resource_path / "meminfo.txt")


@pytest.fixture
def printer_echo_mock(monkeypatch):
    """
    Fixture to mock the Printer function, used in validating compare method outputs
    """
    mock = Mock(return_value=None)
    monkeypatch.setattr(Printer, "echo", lambda *a, **k: mock(*a, **k))
    yield mock


@pytest.fixture
def mock_sys_info():
    """
    Fixture that returns a mock system information object, generated by using the commandline_patches fixture.
    """
    with open(
        Path(__file__).parent
        / "unit"
        / "system_info"
        / "resources"
        / "system_info_expected.json"
    ) as fp:
        sys_info = json.load(fp)
        yield sys_info



from unittest.mock import call

from dell_ai.system_utils.base import Printer
from dell_ai.system_utils.os_info import (
    OSInfo,
    get_os_info,
    get_product_name,
    get_product_prefix,
)
from dell_ai.system_utils import os_info



def test_get_product_name(patched_platform):
    """
    Test product name with patched dmidecode output
    """
    assert get_product_name() == "PowerEdge R760xa"


def test_no_product_name(fp, tmp_path, monkeypatch):
    """
    Test no output when dmidecode output is empty or dmidecode errors
    """
    dmi_file = tmp_path / "product_name"
    monkeypatch.setattr(os_info, "Path", lambda p: dmi_file if "product_name" in p else tmp_path / p)
    fp.register(["hostnamectl", fp.any()], returncode=1, occurrences=2)

    fp.register(["dmidecode", fp.any()], stdout="")
    assert get_product_name() == ""

    fp.register(["dmidecode", fp.any()], returncode=1)
    assert get_product_name() is None


def test_get_product_name_fallback_to_dmi_file(fp, tmp_path, monkeypatch):
    """
    Test fallback to dmi file when dmidecode fails
    """
    fp.register(["dmidecode", fp.any()], returncode=1)
    fp.register(["hostnamectl", fp.any()], returncode=1)

    dmi_file = tmp_path / "product_name"
    dmi_file.write_text("PowerEdge R760xa\n")

    monkeypatch.setattr(os_info, "Path", lambda p: dmi_file if "product_name" in p else tmp_path / p)

    assert get_product_name() == "PowerEdge R760xa"


def test_get_product_name_fallback_to_hostnamectl(fp, tmp_path, monkeypatch):
    """
    Test fallback to hostnamectl when dmidecode and dmi file fail
    """
    import json

    fp.register(["dmidecode", fp.any()], returncode=1)
    fp.register(
        ["hostnamectl", "--json", "short"],
        stdout=json.dumps({"HardwareModel": "PowerEdge XE9680"})
    )

    dmi_file = tmp_path / "product_name"
    monkeypatch.setattr(os_info, "Path", lambda p: dmi_file if "product_name" in p else tmp_path / p)

    assert get_product_name() == "PowerEdge XE9680"


def test_get_product_name_all_sources_fail(fp, tmp_path, monkeypatch):
    """
    Test None returned when all sources fail
    """
    fp.register(["dmidecode", fp.any()], returncode=1)
    fp.register(["hostnamectl", fp.any()], returncode=1)

    dmi_file = tmp_path / "product_name"
    monkeypatch.setattr(os_info, "Path", lambda p: dmi_file if "product_name" in p else tmp_path / p)

    assert get_product_name() is None


def test_get_product_prefix():
    """
    Test prefix name computation based on input
    """
    assert get_product_prefix("PowerEdge R760xa") == "r760xa"
    assert get_product_prefix("Some random name") is None
    assert get_product_prefix(None) is None


def test_get_os_info(patched_platform):
    """
    Test passing os info getter
    """
    assert get_os_info() == OSInfo(
        hostname="deh-r760xaL40-53",
        system="Linux",
        kernel="6.8.0-88-generic",
        linux_distro="ubuntu",
        linux_distro_version="24.04",
        is_linux=True,
        product_name="PowerEdge R760xa",
        product_prefix="r760xa",
    )


def test_os_info_compare_success(printer_echo_mock):
    """
    Test OSInfo comparison success case, when all versions and platform info are tested
    """
    success = OSInfo(
        hostname="node-01",
        system="Linux",
        kernel="6.8.0-42-generic",
        linux_distro="Ubuntu",
        linux_distro_version="22.04",
        is_linux=True,
        product_name="SomeProduct",
        product_prefix="SPX",
    )

    others = [
        OSInfo(
            hostname="node-02",
            system="Linux",
            kernel="6.8.0-42-generic",
            linux_distro="Ubuntu",
            linux_distro_version="22.04",
            is_linux=True,
            product_name="SomeProduct",
            product_prefix="SPX",
        ),
        OSInfo(
            hostname="node-03",
            system="Linux",
            kernel="6.5.0-15-generic",
            linux_distro="Ubuntu",
            linux_distro_version="24.04",
            is_linux=True,
            product_name="AnotherProduct",
            product_prefix="APX",
        ),
    ]

    success.compare(others)

    printer_echo_mock.assert_not_called()


def test_os_info_compare_failure(printer_echo_mock):
    """
    Test failing comparison attributes are not in tested list
    """
    failure = OSInfo(
        hostname="node-01",
        system="Linux",
        kernel="5.10.0-custom", # different kernel
        linux_distro="Debian", # different distro
        linux_distro_version="11", # different version
        is_linux=True,
        product_name="PowerEdge XE9680",
        product_prefix="xe9680",
    )

    others = [
        OSInfo(
            system="Linux",
            kernel="6.8.0-42-generic",
            linux_distro="Ubuntu",
            linux_distro_version="22.04",
            is_linux=True,
            product_name="PowerEdge XE9680",
            product_prefix="xe9680",
        ),
        OSInfo(
            system="Linux",
            kernel="6.5.0-15-generic",
            linux_distro="Fedora",
            linux_distro_version="40",
            is_linux=True,
            product_name="PowerEdge XE9680",
            product_prefix="xe9680",
        ),
    ]

    failure.compare(others)

    calls = []
    for tag, attr_name, self_value, supported_values, level in [
        (
            "Kernel",
            "kernel",
            "5.10.0-custom",
            ["6.8.0-42-generic", "6.5.0-15-generic"],
            "warn",
        ),
        ("Linux Distro", "linux_distro", "Debian", ["Ubuntu", "Fedora"], "error"),
        ("Linux Distro Version", "linux_distro_version", "11", ["22.04", "40"], "info"),
    ]:
        if attr_name == "linux_distro_version":
            calls.append(
                call(
                    Printer.minimum_styled(
                        tag=tag,
                        attr_name=attr_name,
                        self_value=self_value,
                        supported_values=supported_values,
                    ),
                    level=level,
                )
            )
            continue
        calls.append(
            call(
                Printer.list_compare_styled(
                    tag=tag,
                    attr_name=attr_name,
                    self_value=self_value,
                    supported_values=supported_values,
                ),
                level=level,
            )
        )

    printer_echo_mock.assert_has_calls(calls=calls, any_order=True)



from unittest.mock import call

from dell_ai.system_utils import mem_info
from dell_ai.system_utils.base import Printer
from dell_ai.system_utils.mem_info import MemInfo


def test_mem_info(commandline_patches):
    """
    Test mem info object population
    """
    meminfo = mem_info.get_mem_info()
    assert meminfo == mem_info.MemInfo(
        free_kb=1408361652, available_kb=1560040584, hugepages_free_kb=0
    )


def test_mem_info_compare_success(printer_echo_mock):
    """
    Test compare success case
    """
    success = MemInfo(
        free_kb=800_000,
        available_kb=1_600_000,
        hugepages_free_kb=4096,
    )

    others = [
        MemInfo(free_kb=700_000, available_kb=1_500_000, hugepages_free_kb=2048),
        MemInfo(free_kb=900_000, available_kb=1_700_000, hugepages_free_kb=1024),
    ]

    success.compare(others)

    printer_echo_mock.assert_not_called()


def test_mem_info_compare_failure(printer_echo_mock):
    """
    Test compare failure case
    """
    failure = MemInfo(
        free_kb=600_000,
        available_kb=1_400_000,
        hugepages_free_kb=512,
    )

    others = [
        MemInfo(free_kb=700_000, available_kb=1_500_000, hugepages_free_kb=1024),
        MemInfo(free_kb=900_000, available_kb=1_700_000, hugepages_free_kb=2048),
    ]

    failure.compare(others)

    calls = []
    for tag, attr_name, self_value, supported_values in [
        ("Free Memory KB", "free_kb", 600000, [700000, 900000]),
        ("Available Memory KB", "available_kb", 1400000, [1500000, 1700000]),
        ("Hugepages Memory Free KB", "hugepages_free_kb", 512, [1024, 2048]),
    ]:
        calls.append(
            call(
                Printer.minimum_styled(
                    tag=tag,
                    attr_name=attr_name,
                    self_value=self_value,
                    supported_values=supported_values,
                ),
                level="warn",
            )
        )

    printer_echo_mock.assert_has_calls(calls=calls, any_order=True)



from dell_ai.system_utils.system_info import get_system_info


def test_get_system_info(commandline_patches, mock_sys_info):
    """
    Test system info parsing from subprocess patches
    """
    info = get_system_info()
    assert info is not None
    assert info.model_dump() == mock_sys_info



from unittest.mock import call

from dell_ai.system_utils.base import Printer
from dell_ai.system_utils.cpu_info import (
    CPUInfo,
    _recursive_parse_lscpu_out,
    get_cpu_info,
)


def test_recursive_parse_lscpu_out():
    """
    Test the recursive parsing method
    """
    item_list = [
        {"field": "key1", "data": "val1"},
        {
            "field": "key2",
            "data": "val2",
            "children": [{"field": "ckey3", "data": "val3"}],
        },
        {
            "field": "key4",
            "data": None,
            "children": [
                {"field": "ckey5", "data": "val5"},
                {
                    "field": "ckey6",
                    "data": "val6",
                    "children": [
                        {"field": "cckey7", "data": "val7"},
                        {
                            "field": "cckey8",
                            "data": "val8",
                            "children": [{"field": "ccckey9", "data": "val9"}],
                        },
                    ],
                },
            ],
        },
    ]
    expected_response = {
        "key1": "val1",
        "key2": "val2",
        "ckey3": "val3",
        "key4": None,
        "ckey5": "val5",
        "ckey6": "val6",
        "cckey7": "val7",
        "cckey8": "val8",
        "ccckey9": "val9",
    }
    assert expected_response == _recursive_parse_lscpu_out(
        item_list=item_list, parsed_dict={}
    ), "Response does not match"


def test_get_cpu_info_success(commandline_patches):
    """
    Test the get_cpu_info function with subprocess mocks
    """
    cpu_info: CPUInfo | None = get_cpu_info()
    assert cpu_info == CPUInfo(
        cores_per_socket=32,
        threads_per_core=1,
        sockets=2,
        cpu_num=64,
        vendor="GenuineIntel",
    )


def test_cpu_info_compare_success(printer_echo_mock):
    """
    Test the compare function with success cases, if the given 
    configuration is within the supported range of tested configurations.
    """
    success = CPUInfo(
        cores_per_socket=2,
        threads_per_core=2,
        sockets=2,
        cpu_num=2,
        vendor="GenuineIntel",
    )
    others = [
        CPUInfo(
            cores_per_socket=1,
            threads_per_core=1,
            sockets=1,
            cpu_num=1,
            vendor="GenuineIntel",
        ),
        CPUInfo(
            cores_per_socket=5,
            threads_per_core=5,
            sockets=10,
            cpu_num=12,
            vendor="GenuineIntel",
        ),
    ]

    success.compare(others)
    printer_echo_mock.assert_not_called()


def test_cpu_info_compare_failure(printer_echo_mock):
    """
    Test the compare function with failure cases
    """
    failure = CPUInfo(
        cores_per_socket=1,
        threads_per_core=1,
        sockets=2,
        cpu_num=2,
        vendor="GenuineIntel",
    )
    others = [
        CPUInfo(
            cores_per_socket=2,
            threads_per_core=2,
            sockets=1,
            cpu_num=1,
            vendor="GenuineIntel",
        ),
        CPUInfo(
            cores_per_socket=5,
            threads_per_core=5,
            sockets=10,
            cpu_num=12,
            vendor="GenuineIntel",
        ),
    ]
    failure.compare(others)
    calls = []
    for tag, attr_name, self_value, supported_values in [
        ("Cores Per Socket", "cores_per_socket", 1, [2, 5]),
        ("Threads Per Core", "threads_per_core", 1, [2, 5]),
    ]:
        calls.append(
            call(
                Printer.minimum_styled(
                    tag=tag,
                    self_value=self_value,
                    supported_values=supported_values,
                    attr_name=attr_name,
                ),
                level="info",
            )
        )
    printer_echo_mock.assert_has_calls(calls=calls, any_order=True)



import json
from pathlib import Path

from fastapi import FastAPI

app = FastAPI()


@app.get("/auth")
def auth():
    """
    Handles GET requests to the /auth endpoint, returning user authentication information.

    Returns:
        A JSON object containing user details, including username, name, org, canSeeOrg, canSeePrivateRepo, repoSorting, lastLogin, full_name, email, type, id, role, isAnonymous, and isOrganization.
    """
    return {}


resource_file = Path(__file__).parent / "resources" / "sysinfo.json"
resource_folder = Path(__file__).parent / "resources"


@app.get("/skus")
def skus():
    """
    SKU method implementation to illustrate SKU parsing according to the structure in the resources directory
    """
    sku_list = []
    for file_name in resource_folder.iterdir():
        if file_name.is_dir():
            server = file_name.name
            for framework in (resource_folder / server).iterdir():
                if framework.is_dir():
                    if framework.name in ["intel", "nvidia", "amd"]:
                        for sku_name in (
                            resource_folder / server / framework.name
                        ).iterdir():
                            if sku_name.is_dir():
                                sku_list.append(
                                    f"{server.lower()}-{framework.name.lower()}-{sku_name.name.lower()}"
                                )
    return {"skus": sku_list}


@app.get("/skus/{sku_id}")
def sku(sku_id: str):
    """
    Get SKU from sku_id, should be in the resources folder
    
    :param sku_id: Description
    :type sku_id: str
    """
    server, framework, sku_name = sku_id.split("-")
    made_path = resource_folder / server / framework / sku_name
    assert (
        made_path.exists()
    ), f"Did not find server/framework/sku_name combination for {sku_id}"
    assert made_path.is_dir(), "Path should be a folder"
    for filename in made_path.iterdir():
        if filename.is_file() and filename.name.endswith(".json"):
            with open(filename, "r") as fp:
                return json.load(fp)["platform"]
    raise FileNotFoundError(f"{sku_id} not found")


@app.get("/skus/{sku_id}/sysinfo")
def sysinfo(sku_id: str):
    """
    Get sysinfo for the sku_id
    
    :param sku_id: Description
    :type sku_id: str
    """
    server, framework, sku_name = sku_id.split("-")
    sys_infos = []
    made_path = resource_folder / server / framework / sku_name
    assert (
        made_path.exists()
    ), f"Did not find server/framework/sku_name combination for {sku_id}"
    assert made_path.is_dir(), "Path should be a folder"
    for filename in made_path.iterdir():
        if filename.is_file() and filename.name.endswith(".json"):
            with open(filename, "r") as fp:
                sys_infos.append(json.load(fp)["sysinfo"])
    return sys_infos



import itertools
from typing import List

from pydantic import Field
from typing_extensions import Self

from dell_ai.system_utils.base import ComparableBaseModel, Printer
from dell_ai.system_utils.cpu_info import CPUInfo, get_cpu_info
from dell_ai.system_utils.gpu_info import (
    Accelerator,
    AmdDriverInfo,
    GPUInfo,
    IntelDriverInfo,
    NvidiaDriverInfo,
    get_driver_info,
    get_gpus_and_accelerator_info,
)
from dell_ai.system_utils.k8s_info import K8SInfo, get_kube_info
from dell_ai.system_utils.mem_info import MemInfo, get_mem_info
from dell_ai.system_utils.os_info import OSInfo, get_os_info
from dell_ai.system_utils.storage_info import StorageInfo, get_storage_info


class HardwareInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        if self.cpu is not None:
            self.cpu.compare([other.cpu for other in others if other.cpu is not None])
        self.memory.compare([other.memory for other in others])

    cpu: CPUInfo | None
    memory: MemInfo
    storage: StorageInfo


class ContainersAndK8sInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.kubernetes.compare([other.kubernetes for other in others])

    kubernetes: K8SInfo


class ROCMInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        pass

    rocminfo_present: bool = False


class SoftwareInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.containers_and_k8s.compare([other.containers_and_k8s for other in others])
        if self.nvidia is None and self.amd is None and self.intel is None:
            Printer.echo("No Supported GPU entry found", level="error")
        if self.nvidia is not None:
            self.nvidia.compare(
                [other.nvidia for other in others if other.nvidia is not None]
            )
        if self.amd is not None:
            self.amd.compare([other.amd for other in others if other.amd is not None])
        if self.intel is not None:
            self.intel.compare(
                [other.intel for other in others if other.intel is not None]
            )

    amd_rocm: ROCMInfo = Field(
        exclude_if=lambda v: v.rocminfo_present is False,
        default=ROCMInfo(rocminfo_present=False),
    )
    containers_and_k8s: ContainersAndK8sInfo
    nvidia: NvidiaDriverInfo | None = Field(
        exclude_if=lambda v: v is None, default=None
    )
    amd: AmdDriverInfo | None = Field(exclude_if=lambda v: v is None, default=None)
    intel: IntelDriverInfo | None = Field(exclude_if=lambda v: v is None, default=None)


class SystemInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.os.compare([other.os for other in others])
        self.software.compare([other.software for other in others])
        self.hardware.compare([other.hardware for other in others])
        self.accelerators.compare([other.accelerators for other in others])
        # perform GPU comparison
        # 1. Length comparison
        available_gpu_lens = [len(other.gpus) for other in others]
        if len(self.gpus) not in available_gpu_lens:
            Printer.echo(
                f"GPU count {len(self.gpus)} is not tested, supported GPU counts {available_gpu_lens}",
                level="warn",
            )

        # 2. GPU to GPU comparison
        # Ignore if any of them are missing, raise info
        flattened_gpu_list = list(
            itertools.chain.from_iterable([other.gpus for other in others])
        )
        for gpu in self.gpus:
            gpu.compare(flattened_gpu_list)

    os: OSInfo
    software: SoftwareInfo
    hardware: HardwareInfo
    gpus: List[GPUInfo]
    accelerators: Accelerator


def get_system_info() -> SystemInfo | None:
    """Get system information of the system"""
    os_info = get_os_info()

    if os_info.is_linux:
        hardware_info = HardwareInfo(
            cpu=get_cpu_info(), memory=get_mem_info(), storage=get_storage_info()
        )
        gpu_info, accelerator_info = get_gpus_and_accelerator_info()
        rocm_info = ROCMInfo(
            rocminfo_present="amd" in accelerator_info.model_dump().keys()
        )
        driver_info = get_driver_info()
        software_info = SoftwareInfo(
            amd_rocm=rocm_info,
            containers_and_k8s=ContainersAndK8sInfo(kubernetes=get_kube_info()),
            nvidia=driver_info.get("nvidia"),  # type: ignore
            amd=driver_info.get("amd"),  # type: ignore
            intel=driver_info.get("intel"),  # type: ignore
        )
        return SystemInfo(
            os=os_info,
            hardware=hardware_info,
            software=software_info,
            gpus=gpu_info,
            accelerators=accelerator_info,
        )
    return None


if __name__ == "__main__":  # pragma: no cover
    sysinfo = get_system_info()
    if sysinfo is not None:
        print(sysinfo.model_dump_json(indent=2))



from typing import List

from pydantic import BaseModel
from typing_extensions import Self

from dell_ai.system_utils.base import ComparableBaseModel, cmd_stdout


class ChildrenBlockDevice(BaseModel):
    mountpoint: str


class BlockDevice(ComparableBaseModel):
    def compare(self, others: List[Self]):
        pass

    name: str
    size: str
    type: str


class BlockDeviceWithModel(BlockDevice):
    model: str


class ParentBlockDevice(BlockDeviceWithModel):
    children: List[BlockDevice] = []


class LsblkInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        pass

    blockdevices: List[ParentBlockDevice]


class StorageInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        pass

    lsblk: LsblkInfo


def get_storage_info():
    """
    Get relevant storage information of the system
    """
    lsblk_output = cmd_stdout(
        ["lsblk", "-o", "name,model,size,type,mountpoint", "--json"]
    )
    if lsblk_output is None:
        lsblk_output = '{"blockdevices": []}'
    lsblk_info = LsblkInfo.model_validate_json(lsblk_output)
    return StorageInfo(lsblk=lsblk_info)


if __name__ == "__main__":  # pragma: no cover
    print(get_storage_info().model_dump_json(indent=2))



from typing import List

from typing_extensions import Self

from dell_ai.system_utils.base import ComparableBaseModel

MEMINFO_PATH = "/proc/meminfo"


class MemInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.more_than_at_least_one("free_kb", others, "Free Memory KB", level="warn")
        self.more_than_at_least_one(
            "available_kb", others, "Available Memory KB", level="warn"
        )
        self.more_than_at_least_one(
            "hugepages_free_kb", others, "Hugepages Memory Free KB", level="warn"
        )

    free_kb: int | None = None
    available_kb: int | None = None
    hugepages_free_kb: int | None = None


def get_mem_info():
    """
    Returns the current memory information.

    :return: MemInfo
    """
    with open(MEMINFO_PATH) as f:
        meminfo_output = f.read().split("\n")

    meminfo_parsed = {}
    for line in meminfo_output:
        if line == "":
            continue
        line = line.split(":")
        meminfo_parsed[line[0]] = int(line[1].split()[0])

    return MemInfo(
        free_kb=meminfo_parsed.get("MemFree"),
        available_kb=meminfo_parsed.get("MemAvailable"),
        hugepages_free_kb=meminfo_parsed.get("HugePages_Free"),
    )


if __name__ == "__main__":  # pragma: no cover
    print(get_mem_info().model_dump_json(indent=2))



from dell_ai.system_utils.storage_info import get_storage_info


def test_get_storage_info(commandline_patches):
    """
    Test lsblk command output parsing to storage info object
    """
    assert get_storage_info().model_dump() == {
        "lsblk": {
            "blockdevices": [
                {
                    "name": "nvme0n1",
                    "size": "894.2G",
                    "type": "disk",
                    "model": "Dell BOSS-N1",
                    "children": [
                        {"name": "nvme0n1p1", "size": "200G", "type": "part"},
                        {"name": "nvme0n1p2", "size": "20G", "type": "part"},
                        {"name": "nvme0n1p3", "size": "12G", "type": "part"},
                        {"name": "nvme0n1p4", "size": "662.2G", "type": "part"},
                    ],
                },
                {
                    "name": "nvme1n1",
                    "size": "1.7T",
                    "type": "disk",
                    "model": "Dell Ent NVMe CM6 RI 1.92TB",
                    "children": [{"name": "nvme1n1p1", "size": "1.7T", "type": "part"}],
                },
                {
                    "name": "nvme2n1",
                    "size": "1.7T",
                    "type": "disk",
                    "model": "Dell Ent NVMe CM6 RI 1.92TB",
                    "children": [],
                },
            ]
        }
    }



import json
from typing import List

from typing_extensions import Self

from dell_ai.system_utils.base import ComparableBaseModel, cmd_stdout


class CPUInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.more_than_at_least_one("cores_per_socket", others, "Cores Per Socket")
        self.more_than_at_least_one("threads_per_core", others, "Threads Per Core")
        self.more_than_at_least_one("sockets", others, "Sockets")
        self.more_than_at_least_one("cpu_num", others, "Number of CPUs")

    cores_per_socket: int | None = None
    threads_per_core: int | None = None
    sockets: int | None = None
    cpu_num: int | None = None
    vendor: str | None = None


def _recursive_parse_lscpu_out(item_list: List, parsed_dict):
    """
    Recursively parse the lscpu output and return a dictionary containing the parsed data.

    The item list is list of dictionary of the following format:
        {
            "field": "field_name",
            "data": "data",
            "children": [{"field": "child_field", "data": "child_data"}],
        }
    
    Parameters:
    item_list (List): A list of dictionaries containing lscpu output data.
    parsed_dict (dict): A dictionary to store the parsed data.

    Returns:
    dict: A dictionary containing the parsed lscpu output data.
    """
    
    for item in item_list:
        parsed_dict[item["field"]] = item["data"]
        parsed_dict = _recursive_parse_lscpu_out(
            item_list=item.get("children", []), parsed_dict=parsed_dict
        )
    return parsed_dict


def get_cpu_info() -> CPUInfo | None:
    """
    Returns the CPU information of the system. The returned information includes the number of cores per socket, threads per core, sockets, CPU number, and vendor.

    Returns:
    CPUInfo | None: The CPU information of the system, or None if the information cannot be retrieved.
    """
    
    output = cmd_stdout(["lscpu", "--json"])
    if output is None:
        return output
    lscpu_output = json.loads(output)
    lscpu_parsed = _recursive_parse_lscpu_out(lscpu_output["lscpu"], {})
    return CPUInfo(
        cores_per_socket=lscpu_parsed.get("Core(s) per socket:"),
        threads_per_core=lscpu_parsed.get("Thread(s) per core:"),
        sockets=lscpu_parsed.get("Socket(s):"),
        cpu_num=lscpu_parsed.get("CPU(s):"),
        vendor=lscpu_parsed.get("Vendor ID:"),
    )


if __name__ == "__main__":  # pragma: no cover
    try:
        cpu_info_dict = get_cpu_info()
        if cpu_info_dict is not None:
            print(cpu_info_dict.model_dump_json(indent=2))
        else:
            print("Linux CPU info is None")
    except Exception as e:
        print(f"Failed to get Linux CPU info: {e}")



import abc
import subprocess
from typing import Any, List, Literal

import rich
from pydantic import BaseModel
from pydantic_extra_types.semantic_version import SemanticVersion
from typing_extensions import Self


def cmd_stdout(process_args, *args, **kwargs):
    try:
        return subprocess.run(
            process_args, capture_output=True, text=True, check=True, *args, **kwargs
        ).stdout
    except FileNotFoundError:  # command not found
        return None
    except subprocess.CalledProcessError as e:
        rich.print(e)
        return None


class ComparableBaseModel(BaseModel, abc.ABC):
    @abc.abstractmethod
    def compare(self, others: List[Self]):
        pass

    def simple_list_compare(
        self,
        attr_name,
        others: List[Self],
        tag,
        level: Literal["info", "warn", "error"] = "info",
    ):
        supported_values = []
        for other in others:
            value = getattr(other, attr_name)
            if value is not None:
                supported_values.append(value)
        if not supported_values:
            return
        self_value = getattr(self, attr_name)
        if self_value is None:
            Printer.echo(Printer.not_found(tag=tag, attr_name=attr_name), level="error")
            return
        if self_value not in supported_values:
            Printer.echo(
                Printer.list_compare_styled(
                    tag=tag,
                    self_value=self_value,
                    supported_values=supported_values,
                    attr_name=attr_name,
                ),
                level=level,
            )

    def more_than_at_least_one(
        self,
        attr_name,
        others: List[Self],
        tag,
        level: Literal["info", "warn", "error"] = "info",
    ):
        supported_values = []
        for other in others:
            value = getattr(other, attr_name)
            if value is not None:
                supported_values.append(value)
        if not supported_values:
            return
        self_value = getattr(self, attr_name)
        if self_value is None:
            Printer.echo(Printer.not_found(tag=tag, attr_name=attr_name), level="error")
            return
        try:
            if float(self_value) < min([float(x) for x in supported_values]):
                Printer.echo(
                    Printer.minimum_styled(
                        tag=tag,
                        self_value=self_value,
                        supported_values=supported_values,
                        attr_name=attr_name,
                    ),
                    level=level,
                )
        except Exception as e:
            Printer.echo(f"{tag} ({attr_name}) {e}", level="error")

    @classmethod
    def _version_parse(cls, v):
        try:
            return SemanticVersion.parse(v)
        except ValueError as e:
            if "v" in v:
                return cls._version_parse(v.removeprefix("v"))
            if "0" in v:
                major, minor, patch = v.split(".")
                return SemanticVersion(
                    major=int(major), minor=int(minor), patch=int(patch)
                )
            else:
                raise e

    def software_version_compare(
        self,
        attr_name: str,
        others: List[Self],
        tag: str,
    ):
        supported_versions = []
        for other in others:
            value = getattr(other, attr_name)
            if value is not None:
                if isinstance(value, str):
                    try:
                        parsed_value = self._version_parse(value)
                        supported_versions.append(parsed_value)
                    except ValueError:
                        Printer.echo(
                            f"Comparison {tag} ({attr_name}) {value} cannot be parsed as semantic version",
                            level="error",
                        )
                else:
                    Printer.echo(
                        f"Comparison {tag} ({attr_name}) {value} is not a string",
                        level="error",
                    )
        if not supported_versions:
            return
        self_value = getattr(self, attr_name)
        if self_value is None:
            Printer.echo(Printer.not_found(tag=tag, attr_name=attr_name), level="error")
            return
        if isinstance(self_value, str):
            try:
                parsed_self_value = self._version_parse(self_value)
            except ValueError:
                Printer.echo(
                    f"This {tag} ({attr_name}) {self_value} cannot be parsed as semantic version",
                    level="error",
                )
            else:
                min_supported_version = min(supported_versions)
                max_supported_version = max(supported_versions)
                if min_supported_version <= parsed_self_value <= max_supported_version:
                    # found in list of supported versions, nothing to do
                    return
                elif parsed_self_value < min_supported_version:
                    Printer.echo(
                        Printer.version_compare_styled(
                            tag=tag,
                            attr_name=attr_name,
                            self_value=self_value,
                            min_supported_value=str(min_supported_version),
                            max_supported_value=str(max_supported_version),
                            greater=False,
                        ),
                        level="error",
                    )
                elif parsed_self_value > max_supported_version:
                    Printer.echo(
                        Printer.version_compare_styled(
                            tag=tag,
                            attr_name=attr_name,
                            self_value=self_value,
                            min_supported_value=str(min_supported_version),
                            max_supported_value=str(max_supported_version),
                            greater=True,
                        ),
                        level="warn",
                    )
        else:
            Printer.echo(
                f"This {tag} ({attr_name}) {self_value} is not a string", level="error"
            )
            return


class Printer:
    @classmethod
    def echo(cls, message: str, level: Literal["info", "warn", "error"] = "info"):
        rich.print(cls.styled(message, level=level))

    @classmethod
    def styled(
        cls, message: str, level: Literal["info", "warn", "error"] = "info"
    ) -> str:
        if level == "warn":
            return f":warning: [yellow]WARNING: {message} [/yellow]"
        elif level == "error":
            return f":red_circle: [red]ERROR: {message} [/red]"
        return f":grey_exclamation: [green]INFO: {message} [/green]"

    @classmethod
    def list_compare_styled(
        cls, self_value: Any, supported_values: List[Any], tag: str, attr_name: str
    ) -> str:
        return f"[italics]{tag}[/italics] {attr_name}=[bold]{self_value}[/bold] not found in supported values: {supported_values}"

    @classmethod
    def minimum_styled(
        cls,
        self_value: int | float | str,
        supported_values: List[int | float | str],
        tag: str,
        attr_name: str,
    ) -> str:
        return f"[italics]{tag}[/italics] {attr_name}=[bold]{self_value}[/bold] lesser than supported values: {supported_values}"

    @classmethod
    def not_found(cls, tag, attr_name):
        return f"[italics]{tag}[/italics] {attr_name} not found!"

    @classmethod
    def version_compare_styled(
        cls,
        self_value: str,
        tag: str,
        attr_name: str,
        min_supported_value: str,
        max_supported_value: str,
        greater: bool = True,
    ) -> str:
        if greater:
            return f"[italics]{tag}[/italics] {attr_name}=[bold]{self_value}[/bold] greater than max supported value: {max_supported_value}"
        else:
            return f"[italics]{tag}[/italics] {attr_name}=[bold]{self_value}[/bold] lesser than min supported value: {min_supported_value}"



import json
from typing import List, Set

from pydantic_extra_types.semantic_version import SemanticVersion
from typing_extensions import Self

from dell_ai.system_utils.base import ComparableBaseModel, Printer, cmd_stdout


class K8SInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.software_version_compare(
            "server_version", others, "Kubernetes Server Version"
        )
        self.simple_list_compare(
            "server_platform", others, "Kubernetes Platform Version", level="error"
        )
        other_kubelet_versions: Set[str] = set()
        try:
            for other in others:
                for kubelet_version in other.node_kubelet_version:
                    other_kubelet_versions.add(
                        str(SemanticVersion.parse(kubelet_version.removeprefix("v")))
                    )
            if not other_kubelet_versions:
                Printer.echo("No other tested kubelet versions found", level="warn")
                return
            min_kubelet_version = min(other_kubelet_versions)
            max_kubelet_version = max(other_kubelet_versions)
            for node_version in set(self.node_kubelet_version):
                parsed_node_version = SemanticVersion.parse(
                    node_version.removeprefix("v")
                )
                if parsed_node_version < min_kubelet_version:
                    Printer.echo(
                        f"Node Kubelet version {node_version} is lower than minimum supported kubelet version {min_kubelet_version}",
                        level="error",
                    )
                elif parsed_node_version > max_kubelet_version:
                    Printer.echo(
                        f"Node Kubelet version {node_version} is higher than maximum supported kubelet version {max_kubelet_version}",
                        level="warn",
                    )
        except ValueError as e:
            Printer.echo(f"Failed to parse kubelet version: {e}", level="error")

    server_version: str | None = None
    server_platform: str | None = None
    node_kubelet_version: List[str] = []


def get_kube_info() -> K8SInfo:
    """
    Returns information about the Kubernetes cluster.

    Returns:
        K8SInfo: An object containing server version, server platform, and node kubelet version.
    """
    kubelet_server_info = cmd_stdout(["kubectl", "version", "-o", "json"]) or "{}"
    kubelet_server_info_parsed = json.loads(kubelet_server_info).get(
        "serverVersion", {}
    )

    kubelet_info = cmd_stdout(["kubectl", "get", "nodes", "-o", "json"]) or "{}"
    kubelet_info_parsed = json.loads(kubelet_info).get("items", [])
    kubelet_versions = {
        node["status"]["nodeInfo"]["kubeletVersion"] for node in kubelet_info_parsed
    }
    return K8SInfo(
        server_version=kubelet_server_info_parsed.get("gitVersion"),
        server_platform=kubelet_server_info_parsed.get("platform"),
        node_kubelet_version=sorted(list(kubelet_versions)),
    )


if __name__ == "__main__":  # pragma: no cover
    print(get_kube_info().model_dump_json(indent=2))



import json
import logging
from pathlib import Path
import platform
from typing import List

from typing_extensions import Self

from dell_ai.system_utils.base import ComparableBaseModel, cmd_stdout

logger = logging.getLogger(__name__)

DMI_FILE_PATH = "/sys/class/dmi/id/product_name"

class OSInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.simple_list_compare("kernel", others, "Kernel", level="warn")
        self.simple_list_compare("linux_distro", others, "Linux Distro", level="error")
        self.more_than_at_least_one(
            "linux_distro_version", others, "Linux Distro Version"
        )

    hostname: str | None = None
    system: str
    kernel: str
    linux_distro: str | None
    linux_distro_version: str | None
    is_linux: bool
    product_name: str | None
    product_prefix: str | None


def get_product_name_from_hostnamectl():
    """
    Get the product name from hostnamectl
    """
    hostnamectl_output = cmd_stdout(["hostnamectl", "--json", "short"])
    if hostnamectl_output is not None:
        hardware_model = json.loads(hostnamectl_output).get("HardwareModel")
        if hardware_model is not None:
            return hardware_model.strip()


def get_product_name_from_dmi():
    """
    Get the product name from dmidecode
    """
    prod_name = cmd_stdout(["dmidecode", "-s", "system-product-name"])
    if prod_name is not None:
        return prod_name.strip()
    else:
        logger.warning(
            "dell-ai utils describe-system/check-system works without sudo, "
            "but elevated privileges may improve hardware identification on some systems."
        )


def get_product_name_from_dmi_file():
    """
    Get the product name from dmi file
    """
    try:
        path = Path(DMI_FILE_PATH)
        if path.exists():
            info = path.read_text().strip()
            if info:
                return info
    except Exception as e:
        logger.info(f"Failed to get product name from dmi file: {e}")
        return None


def get_product_name() -> str | None:
    """Get the product name of the system"""
    sources = [
        get_product_name_from_dmi_file,
        get_product_name_from_hostnamectl,
        get_product_name_from_dmi,
    ]
    for source in sources:
        prod_name = source()
        if prod_name is not None:
            return prod_name.strip()
    return None


def get_product_prefix(prod_name: str | None) -> str | None:
    """
    Get the product prefix of the system
    Format of prod_name should be 'PowerEdge <systemname>'
    """
    if prod_name is None:
        return None
    if prod_name.startswith("PowerEdge"):
        return prod_name.removeprefix("PowerEdge").strip().lower()
    return None


def get_os_info():
    """Get the OS information of the system"""
    uname = platform.uname()
    try:
        os_release = platform.freedesktop_os_release()
    except:
        os_release = {}
    product_name = get_product_name()
    product_prefix = get_product_prefix(product_name)

    return OSInfo(
        hostname=uname.node,
        system=uname.system,
        kernel=uname.release,
        linux_distro=os_release.get("ID"),
        linux_distro_version=os_release.get("VERSION_ID"),
        is_linux=uname.system == "Linux",
        product_name=product_name,
        product_prefix=product_prefix,
    )


if __name__ == "__main__":  # pragma: no cover
    print(get_os_info().model_dump_json(indent=2))



import json
import logging
import re
import platform
from abc import abstractmethod
from typing import Dict, List, Literal, Tuple, Union

from pydantic import RootModel
from typing_extensions import Self

from dell_ai.system_utils.base import ComparableBaseModel, Printer, cmd_stdout

logger = logging.getLogger(__name__)


class NvidiaDriverInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.more_than_at_least_one(
            "cuda_version_from_nvidia_smi",
            others,
            "CUDA version from NVIDIA SMI",
            level="error",
        )
        self.software_version_compare("driver_version", others, "Driver version")
        self.software_version_compare(
            "nvidia_container_toolkit_version",
            others,
            "NVIDIA Container Toolkit version",
        )
        self.software_version_compare(
            "nvidia_ctk_version", others, "NVIDIA CTK version"
        )

    cuda_version_from_nvidia_smi: str | None = None
    driver_version: str | None = None
    nvidia_container_toolkit_version: str | None = None
    nvidia_ctk_version: str | None = None


class AmdDriverInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.more_than_at_least_one(
            "cuda_version_from_rocm_smi",
            others,
            "CUDA version from ROCM SMI",
            level="error",
        )
        self.software_version_compare("driver_version", others, "Driver version")

    cuda_version_from_rocm_smi: str | None = None
    driver_version: str | None = None


class IntelDriverInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.more_than_at_least_one(
            "driver_version", others, "Driver version", level="error"
        )

    driver_version: str | None = None


class AcceleratorInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        self.software_version_compare("driver_version", others, "Driver version")

    driver_version: str | None = None
    name: str | None = None


class Accelerator(RootModel):
    root: Dict[Literal["nvidia", "amd", "intel"], List[AcceleratorInfo]]

    def __iter__(self):
        return iter(self.root.items())

    def __getitem__(self, item):
        return self.root[item]

    def __contains__(self, item):
        return item in self.root.keys()

    def compare(self, others: List[Self]):
        """
        Compare against others by filtering against the root key, so that nvidia accelerator is
        only compared against other nvidia accelerator infos.
        
        Params:
            others (List): Other Accelerator objects
        """
        if len(self.root.keys()) > 1:
            logger.error(f"Found more than one root key {list(self.root.keys())}")
        else:
            _main_key = list(self.root.keys())[0]
            vals = []
            other_keys = []
            for other in others:
                for key in other.root:
                    if key not in other_keys:
                        other_keys.append(key)
                if _main_key in other.root:
                    val = other.root[_main_key]
                    vals.extend(val)
            if _main_key not in other_keys:
                Printer.echo(
                    Printer.list_compare_styled(
                        tag="Accelerator",
                        self_value=_main_key,
                        supported_values=other_keys,
                        attr_name="info",
                    ),
                    level="error",
                )
                return
            for accel_info in self.root.values():
                for accel in accel_info:
                    accel.compare(vals)


class GPUInfo(ComparableBaseModel):
    def compare(self, others: List[Self]):
        other_vendors = list(
            {other.vendor for other in others if other.vendor is not None}
        )
        relevant_others = [other for other in others if other.vendor == self.vendor]
        if not relevant_others:
            Printer.echo(
                f"Found no supported vendor configuration for vendor {self.vendor}, supported {other_vendors}",
                level="error",
            )
            return
        self.simple_list_compare("model", relevant_others, "Model", level="info")
        self.more_than_at_least_one("ram", relevant_others, "GPU RAM", level="warn")
        self.software_version_compare(
            "driver_version", relevant_others, "Driver Version"
        )
        self.more_than_at_least_one(
            "compute_cap", relevant_others, "Compute Cap", level="error"
        )

    vendor: Literal["NVIDIA", "AMD", "INTEL"] | None = None
    model: str | None = None
    # count: int | None = None
    ram: int | None = None
    driver_version: str | None = None
    compute_cap: int | None = None
    index: int | None = None


class GPUInfoPopulater:
    """
    Abstract class for populating GPU info in the details property
    """
    def __init__(self) -> None:
        self.details: Union[NvidiaDriverInfo, AmdDriverInfo, IntelDriverInfo] = (
            NvidiaDriverInfo()
        )
        self.collect_gpu_info()

    @abstractmethod
    def collect_gpu_info(self):
        raise NotImplementedError()

    def get_software_driver_info(
        self,
    ) -> Union[NvidiaDriverInfo, AmdDriverInfo, IntelDriverInfo]:
        return self.details


class AMDInfoPopulater(GPUInfoPopulater):
    # not implemented
    pass


class IntelInfoPopulater(GPUInfoPopulater):
    # not implemented
    pass


class NvidiaInfoPopulater(GPUInfoPopulater):
    NVIDIA_SMI_REGEX = r"CUDA Version: ([\d\.]+)"
    DRIVER_REGEX = r"Driver Version: (\d+\.\d+\.\d+)"
    NVIDIA_CTK_REGEX = r"NVIDIA Container Toolkit CLI version (\d+\.\d+\.\d+)"
    NVIDIA_CONTAINER_TOOLKIT_REGEX = (
        r"NVIDIA Container Runtime Hook version (\d+\.\d+\.\d+)"
    )
    KUBECTL_CTK_REGEX = r"nvcr.io/nvidia/k8s/container-toolkit:v(\d+\.\d+\.\d+).+"

    def __init__(self) -> None:
        super().__init__()
        self.details: NvidiaDriverInfo = NvidiaDriverInfo()
        self.collect_gpu_info()

    def collect_gpu_info(self):
        self.smi_get_cuda()
        self.get_ctk_version()
        self.get_nvidia_toolkit_version()

    def smi_get_cuda(self):
        """
        From nvidia-smi output, obtain CUDA version and driver version
        """
        nvidia_smi_out = cmd_stdout(["nvidia-smi"])
        kubectl_labels = None
        if nvidia_smi_out is not None:
            # use regex to parse
            match = re.search(self.NVIDIA_SMI_REGEX, nvidia_smi_out)
        else:
            match = re.search(self.NVIDIA_SMI_REGEX, "")
            output = cmd_stdout(["kubectl", "get", "nodes", "-o", "json"])
            if output is not None:
                kubectl_output = json.loads(output)
                # Extract labels from the node matching system hostname
                system_hostname = platform.uname().node.lower()
                for item in kubectl_output.get("items", []):
                    if item.get("metadata", {}).get("name", "").lower() == system_hostname:
                        kubectl_labels = item.get("metadata", {}).get("labels", {})
                        break
        if match is not None:
            self.details.cuda_version_from_nvidia_smi = match.group(1)
        else:
            if kubectl_labels is None:
                return
            cuda_version = kubectl_labels.get("nvidia.com/cuda.runtime-version.full")
            if cuda_version is not None:
                self.details.cuda_version_from_nvidia_smi = cuda_version
        if nvidia_smi_out is not None:
            match = re.search(self.DRIVER_REGEX, nvidia_smi_out)
        else:
            match = re.search(self.DRIVER_REGEX, "")
        if match is not None:
            self.details.driver_version = match.group(1)
        else:
            if kubectl_labels is None:
                return
            driver_version = kubectl_labels.get("nvidia.com/cuda.driver-version.full")
            if driver_version is not None:
                self.details.driver_version = driver_version

    def get_ctk_version(self):
        """
        Get Nvidia CTK version
        """
        # try nvidia-ctk
        ctk_version = self.nvidia_ctk_version()
        # if nvidia-ctk not found, try kubectl get node
        if ctk_version is None:
            ctk_version = self.kubectl_ctk_version()
        self.details.nvidia_ctk_version = ctk_version

    def get_nvidia_toolkit_version(self):
        """
        Populate Nvidia container toolkit version
        """
        self.details.nvidia_container_toolkit_version = (
            self.nvidia_container_toolkit_version()
        )

    def nvidia_ctk_version(self):
        """
        Use nvidia-ctk CLI tool to get CTK version
        """
        output = cmd_stdout(["nvidia-ctk", "--version"])
        if output is None:
            return None
        match = re.search(self.NVIDIA_CTK_REGEX, output.splitlines()[0])
        if match:
            return match.group(1)
        return None

    def nvidia_container_toolkit_version(self):
        """
        Use container runtime hook CLI tool to get version
        """
        output = cmd_stdout(["/usr/bin/nvidia-container-runtime-hook", "--version"])
        if output is None:
            return None
        match = re.search(self.NVIDIA_CONTAINER_TOOLKIT_REGEX, output.splitlines()[0])
        if match:
            return match.group(1)
        return None

    def kubectl_ctk_version(self):
        """
        Find Nvidia CTK version from node info
        """
        output = cmd_stdout(["kubectl", "get", "nodes", "-o", "json"])
        if output is None:
            return None
        match = re.search(self.KUBECTL_CTK_REGEX, output)
        if match:
            return match.group(1)
        return None


class NvidiaInfoGetter:
    def __init__(self):
        self.gpu_info: List[GPUInfo] = []
        self.accelerator_info: List[AcceleratorInfo] = []
        self.collect_gpu_info()

    def collect_gpu_info(self):
        """
        Use nvidia-smi to obtain driver version, memory and compute cap of all GPUs
        """
        gpus = cmd_stdout(
            [
                "nvidia-smi",
                "--query-gpu=name,driver_version,memory.total,compute_cap",
                "--format=csv,noheader",
            ]
        )
        if gpus is None:
            output = cmd_stdout(["kubectl", "get", "nodes", "-o", "json"])
            kubectl_labels = None
            if output is not None:
                kubectl_output = json.loads(output)
                # Extract labels from the node matching system hostname
                system_hostname = platform.uname().node.lower()
                for item in kubectl_output.get("items", []):
                    if item.get("metadata", {}).get("name", "").lower() == system_hostname:
                        kubectl_labels = item.get("metadata", {}).get("labels")
                        break
            if kubectl_labels is None:
                return
            for i in range(int(kubectl_labels.get("nvidia.com/gpu.count", 0))):
                gpu_info = GPUInfo(
                    vendor="NVIDIA",
                    index=i,
                    model=kubectl_labels.get("nvidia.com/gpu.product").replace("-", " "), # since the product name from NVIDIA SMI has a space, and product name in kubectl doesn't 
                    driver_version=kubectl_labels.get("nvidia.com/cuda.driver-version.full"),
                    ram=int(kubectl_labels.get("nvidia.com/gpu.memory", 0)),
                    compute_cap=int(f"{kubectl_labels.get('nvidia.com/gpu.compute.major', 0)}{kubectl_labels.get('nvidia.com/gpu.compute.minor', 0)}"),
                )
                self.gpu_info.append(gpu_info)
                accelerator_info = AcceleratorInfo(
                    driver_version=kubectl_labels.get("nvidia.com/cuda.driver-version.full"), name=kubectl_labels.get("nvidia.com/gpu.product")
                )
                self.accelerator_info.append(accelerator_info)

            return 
            
        gpus = gpus.splitlines()

        for i, gpu in enumerate(gpus):
            values = gpu.split(",")
            gpu_info = GPUInfo(
                vendor="NVIDIA",
                index=i,
                model=values[0].strip(),
                driver_version=values[1].strip(),
                ram=int(values[2].removesuffix("MiB").strip()),
                compute_cap=int(values[3].replace(".", "")),
            )
            self.gpu_info.append(gpu_info)
            accelerator_info = AcceleratorInfo(
                driver_version=values[1].strip(), name=values[0].strip()
            )
            self.accelerator_info.append(accelerator_info)

    def get_gpu_info(self) -> List[GPUInfo]:
        return self.gpu_info

    def get_accelerator_info(self) -> List[AcceleratorInfo]:
        return self.accelerator_info


class GPUInfoGetter:
    """
    Collated GPU info getter that checks which GPU is present and returns the information for that GPU type
    """
    VENDOR_MAP = {
        "10de": "NVIDIA",
        "1002": "AMD",
        "8086": "INTEL",
    }
    CLASS_CODES = ["0302", "1200"]

    def __init__(self):
        self.vendors = self.get_gpu_vendors()

    @classmethod
    def get_gpu_vendors(cls) -> list[str]:
        """
        Get GPU vendor type to define which type of output to return
        """
        vendors = set()
        output = cmd_stdout(["lspci", "-nn"])
        if output is None:
            return []
        lines = output.splitlines()
        for line in lines:
            # Check if class code matches GPU/accelerator
            if any(f"[{code}]" in line for code in cls.CLASS_CODES):
                # For each vendor id, check if current line matches
                for vendor_id in cls.VENDOR_MAP:
                    if f"[{vendor_id}:" in line:
                        # Add corresponding vendor to the set
                        vendors.add(cls.VENDOR_MAP[vendor_id])
        return sorted(list(vendors))

    def get_gpu_accelerator(self) -> Tuple[List[GPUInfo], Accelerator]:
        """
        On the basis of vendor, return GPU and accelerator information
        """
        if "NVIDIA" in self.vendors:
            info_getter = NvidiaInfoGetter()
            gpus = info_getter.get_gpu_info()
            accelerators = info_getter.get_accelerator_info()
            return gpus, Accelerator.model_validate({"nvidia": accelerators})
        elif "AMD" in self.vendors:
            Printer.echo("AMD GPUs found, but not supported yet", level="warn")
            return [], Accelerator.model_validate({"amd": []})
        elif "INTEL" in self.vendors:
            Printer.echo("Intel GPUs found, but not supported yet", level="warn")
            return [], Accelerator.model_validate({"intel": []})
        else:
            Printer.echo("No GPUs found, not supported", level="warn")
            return [], Accelerator.model_validate({})

    def get_software_details(
        self,
    ) -> Dict[
        Literal["nvidia", "amd", "intel"],
        Union[NvidiaDriverInfo, AmdDriverInfo, IntelDriverInfo],
    ]:
        """
        Get software info according to vendor type
        """
        ret_dict: Dict[
            Literal["nvidia", "amd", "intel"],
            Union[NvidiaDriverInfo, AmdDriverInfo, IntelDriverInfo],
        ] = {}
        if "NVIDIA" in self.vendors:
            info_populater = NvidiaInfoPopulater()
            ret_dict["nvidia"] = info_populater.get_software_driver_info()
        elif "AMD" in self.vendors:
            pass # to be implemented later, warning has already been raised
        elif "INTEL" in self.vendors:
            pass # to be implemented later, warning has already been raised
        return ret_dict


def get_gpus_and_accelerator_info():
    """
    Aggregated getter for GPU and accelerator information. Returns a tuple of GPUInfo and Accelerator
    """
    return GPUInfoGetter().get_gpu_accelerator()


def get_driver_info():
    """
    Aggregated getter for driver information.
    """
    return GPUInfoGetter().get_software_details()


if __name__ == "__main__":  # pragma: no cover
    gpus, accelerators = get_gpus_and_accelerator_info()
    [print(gpu.model_dump_json(indent=2)) for gpu in gpus]
    print(accelerators.model_dump_json(indent=2))
    for vendor, driver_info in get_driver_info().items():
        print(f"{vendor}: {driver_info.model_dump_json(indent=2)}")



from pathlib import Path
from unittest.mock import call

import pytest

from dell_ai.system_utils.base import Printer
from dell_ai.system_utils.gpu_info import (
    Accelerator,
    AcceleratorInfo,
    AmdDriverInfo,
    GPUInfo,
    GPUInfoGetter,
    NvidiaDriverInfo,
    NvidiaInfoPopulater,
    get_driver_info,
    get_gpus_and_accelerator_info,
)

resource_folder = Path(__file__).parent / "resources"


@pytest.fixture
def lspci(fp):
    """
    Fixture to mock the lspci command for nvidia gpus
    """
    lspci_nvidia_out = resource_folder / "lspci_stdout_nvidia_gpu.txt"
    fp.register(["lspci", "-nn"], stdout=lspci_nvidia_out.read_text())
    yield


def test_get_gpu_vendors_pass(lspci):
    """
    Test vendor discovery using lscpi fixture
    """
    vendors = GPUInfoGetter.get_gpu_vendors()
    assert "NVIDIA" in vendors, "GPU entries are present"


def test_gpu_vendors_fail(fp):
    """
    Do not get any GPU vendor if lspci errors out
    """
    fp.register(["lspci", fp.any()], returncode=1)
    assert GPUInfoGetter.get_gpu_vendors() == []


def test_get_gpu_accelerator_fail(lspci, fp):
    """
    Test gpu and accelarator info is empty if nvidia-smi does not exist
    """
    fp.register(
        ["nvidia-smi", fp.any()], returncode=1, stdout="nvidia-smi: Command not found"
    )
    fp.register(["kubectl", fp.any()], returncode=1, occurrences=4)
    gpu, accelerators = GPUInfoGetter().get_gpu_accelerator()
    assert (gpu, accelerators.model_dump()) == ([], {"nvidia": []})


@pytest.fixture
def lspci_intel(fp):
    """
    Fixture to mock the lspci command for intel accelerators.
    """
    intel_output = (
        "00:02.0 3D controller [0302]: Intel Corporation Device [8086:56c0]\n"
    )
    fp.register(["lspci", "-nn"], stdout=intel_output)
    yield


def test_get_gpu_accelerator_intel_branch(lspci_intel):
    """
    Test intel vendor is routed to intel accelerator branch.
    """
    gpu, accelerators = GPUInfoGetter().get_gpu_accelerator()
    print(gpu, accelerators)
    assert (gpu, accelerators.model_dump()) == ([], {"intel": []})


def test_get_gpu_accelerator_pass(commandline_patches):
    """
    With the patched CLI commands, test success case of GPU and accelerator information. 
    Tests the collect_gpu_info function
    """
    gpus, accelerators = get_gpus_and_accelerator_info()
    assert gpus == [
        GPUInfo(
            vendor="NVIDIA",
            model="NVIDIA L40S",
            ram=46068,
            driver_version="570.172.08",
            compute_cap=89,
            index=0,
        ),
        GPUInfo(
            vendor="NVIDIA",
            model="NVIDIA L40S",
            ram=46068,
            driver_version="570.172.08",
            compute_cap=89,
            index=1,
        ),
        GPUInfo(
            vendor="NVIDIA",
            model="NVIDIA L40S",
            ram=46068,
            driver_version="570.172.08",
            compute_cap=89,
            index=2,
        ),
        GPUInfo(
            vendor="NVIDIA",
            model="NVIDIA L40S",
            ram=46068,
            driver_version="570.172.08",
            compute_cap=89,
            index=3,
        ),
    ]
    assert accelerators.model_dump() == {
        "nvidia": [
            dict(driver_version="570.172.08", name="NVIDIA L40S"),
            dict(driver_version="570.172.08", name="NVIDIA L40S"),
            dict(driver_version="570.172.08", name="NVIDIA L40S"),
            dict(driver_version="570.172.08", name="NVIDIA L40S"),
        ]
    }


def test_nvidia_info_populator(commandline_patches):
    """
    Test all getters in the NvidiaInfoPopulater class
    """
    info = NvidiaInfoPopulater()
    assert info.details.cuda_version_from_nvidia_smi == "12.8"
    assert info.details.nvidia_ctk_version == "1.17.8"
    assert info.details.driver_version == "570.172.08"


def test_smi_get_cuda_success(fp):
    """
    Test smi_get_cuda parses CUDA version and driver version from nvidia-smi output
    """
    nvidia_smi_out = resource_folder / "nvidia_smi.txt"
    fp.register(["nvidia-smi"], stdout=nvidia_smi_out.read_text(), occurrences=2)
    fp.register(["kubectl", "get", "nodes", "-o", "json"], returncode=1, occurrences=4)
    fp.register(["nvidia-ctk", fp.any()], returncode=1, occurrences=2)
    fp.register(["/usr/bin/nvidia-container-runtime-hook", fp.any()], returncode=1, occurrences=2)

    info = NvidiaInfoPopulater()
    assert info.details.cuda_version_from_nvidia_smi == "12.8"
    assert info.details.driver_version == "570.172.08"

def test_smi_get_cuda_nvidia_smi_fails(fp):
    """
    Test smi_get_cuda returns early when nvidia-smi command fails
    """
    fp.register(["nvidia-smi"], returncode=1, stdout="nvidia-smi: Command not found", occurrences=2)
    fp.register(["kubectl", "get", "nodes", "-o", "json"], returncode=1, occurrences=4)
    fp.register(["nvidia-ctk", fp.any()], returncode=1, occurrences=2)
    fp.register(["/usr/bin/nvidia-container-runtime-hook", fp.any()], returncode=1, occurrences=2)

    info = NvidiaInfoPopulater()
    assert info.details.cuda_version_from_nvidia_smi is None
    assert info.details.driver_version is None

def test_smi_get_cuda_partial_output(fp):
    """
    Test smi_get_cuda when nvidia-smi output has CUDA version but not driver version
    """
    partial_output = "CUDA Version: 12.5"
    fp.register(["nvidia-smi"], stdout=partial_output, occurrences=2)
    fp.register(["kubectl", "get", "nodes", "-o", "json"], returncode=1, occurrences=4)
    fp.register(["nvidia-ctk", fp.any()], returncode=1, occurrences=2)
    fp.register(["/usr/bin/nvidia-container-runtime-hook", fp.any()], returncode=1, occurrences=2)

    info = NvidiaInfoPopulater()
    assert info.details.cuda_version_from_nvidia_smi == "12.5"
    assert info.details.driver_version is None

def test_smi_get_cuda_driver_only(fp):
    """
    Test smi_get_cuda when nvidia-smi output has driver version but not CUDA version.
    Note: When CUDA regex doesn't match and kubectl fails, the method returns early
    before checking driver regex (expected behavior per current implementation).
    """
    partial_output = "Driver Version: 535.104.05"
    fp.register(["nvidia-smi"], stdout=partial_output, occurrences=2)
    fp.register(["kubectl", "get", "nodes", "-o", "json"], returncode=1, occurrences=4)
    fp.register(["nvidia-ctk", fp.any()], returncode=1, occurrences=2)
    fp.register(["/usr/bin/nvidia-container-runtime-hook", fp.any()], returncode=1, occurrences=2)

    info = NvidiaInfoPopulater()
    # Both are None because CUDA regex doesn't match and kubectl fails,
    # so the method returns early before checking driver regex
    assert info.details.cuda_version_from_nvidia_smi is None
    assert info.details.driver_version is None

def test_smi_get_cuda_no_regex_match(fp):
    """
    Test smi_get_cuda when nvidia-smi output doesn't match any regex patterns
    """
    invalid_output = "Some random output without version info"
    fp.register(["nvidia-smi"], stdout=invalid_output, occurrences=2)
    fp.register(["kubectl", "get", "nodes", "-o", "json"], returncode=1, occurrences=4)
    fp.register(["nvidia-ctk", fp.any()], returncode=1, occurrences=2)
    fp.register(["/usr/bin/nvidia-container-runtime-hook", fp.any()], returncode=1, occurrences=2)

    info = NvidiaInfoPopulater()
    assert info.details.cuda_version_from_nvidia_smi is None
    assert info.details.driver_version is None

def test_smi_get_cuda_different_versions(fp):
    """
    Test smi_get_cuda with various CUDA and driver version formats
    """
    custom_output = "+-----------------------------------------------------------------------------------------+\n| NVIDIA-SMI 545.23.06              Driver Version: 545.23.06     CUDA Version: 12.3     |\n"
    fp.register(["nvidia-smi"], stdout=custom_output, occurrences=2)
    fp.register(["kubectl", "get", "nodes", "-o", "json"], returncode=1, occurrences=4)
    fp.register(["nvidia-ctk", fp.any()], returncode=1, occurrences=2)
    fp.register(["/usr/bin/nvidia-container-runtime-hook", fp.any()], returncode=1, occurrences=2)

    info = NvidiaInfoPopulater()
    assert info.details.cuda_version_from_nvidia_smi == "12.3"
    assert info.details.driver_version == "545.23.06"


def test_nvidia_info_populator_no_nvidia(fp):
    """
    Test getters if driver is not installed correctly and CLI tools are missing
    """
    fp.register(
        ["nvidia-smi"],
        returncode=1,
        stdout="nvidia-smi: Command not found",
        occurrences=2,
    )
    fp.register(
        ["nvidia-ctk", fp.any()],
        returncode=1,
        stdout="nvidia-ctk: Command not found",
        occurrences=2,
    )
    fp.register(["kubectl", fp.any()], returncode=1, occurrences=4)
    fp.register(
        ["/usr/bin/nvidia-container-runtime-hook", fp.any()],
        returncode=1,
        stdout="/usr/bin/nvidia-container-runtime-hook: File not found",
        occurrences=2,
    )
    info = NvidiaInfoPopulater()
    assert info.details.cuda_version_from_nvidia_smi is None
    assert info.details.driver_version is None
    assert info.details.nvidia_ctk_version is None
    assert info.details.nvidia_container_toolkit_version is None


def test_nvidia_info_populator_k8s_info(fp):
    """
    Test fallback on kubectl if CLI tools are missing
    """
    fp.register(
        ["nvidia-smi"],
        returncode=1,
        stdout="nvidia-smi: Command not found",
        occurrences=2,
    )
    fp.register(
        ["nvidia-ctk", fp.any()],
        returncode=1,
        stdout="nvidia-ctk: Command not found",
        occurrences=2,
    )
    fp.register(
        ["kubectl", "get", "nodes", "-o", "json"],
        stdout=(resource_folder / "kubectl_get_nodes.json").read_text(),
        occurrences=4,
    )
    fp.register(
        ["/usr/bin/nvidia-container-runtime-hook", fp.any()],
        returncode=1,
        stdout="/usr/bin/nvidia-container-runtime-hook: File not found",
        occurrences=2,
    )
    info = NvidiaInfoPopulater()
    assert info.details.cuda_version_from_nvidia_smi is None
    assert info.details.driver_version is None
    assert info.details.nvidia_ctk_version == "1.17.8"
    assert info.details.nvidia_container_toolkit_version is None


def test_get_driver_info(commandline_patches):
    """
    Test model assignment. Should be of type 
        {
            "nvidia": NvidiaDriverInfo(...)
        }
    for nvidia and similar for amd and intel. Those tests will be implemented later
    """
    info = get_driver_info()
    assert info == {
        "nvidia": NvidiaDriverInfo.model_validate(
            {
                "cuda_version_from_nvidia_smi": "12.8",
                "nvidia_ctk_version": "1.17.8",
                "driver_version": "570.172.08",
                "nvidia_container_toolkit_version": "1.17.8",
            }
        )
    }


def test_nvidia_driver_info_compare(printer_echo_mock):
    """
    Tests the comparison of NvidiaDriverInfo objects.

    This function creates several NvidiaDriverInfo objects with different versions
    of CUDA, driver, and NVIDIA Container Toolkit, and then compares them to test
    the compare method of the NvidiaDriverInfo class.

    """
    # all versions in tested range
    success = NvidiaDriverInfo(
        cuda_version_from_nvidia_smi="12.8",
        driver_version="566.125.15",
        nvidia_container_toolkit_version="17.8.1",
    )
    others = [
        NvidiaDriverInfo(
            cuda_version_from_nvidia_smi="12.8",
            driver_version="594.564.56",
            nvidia_container_toolkit_version="17.8.1",
        ),
        NvidiaDriverInfo(
            cuda_version_from_nvidia_smi="13.0", driver_version="566.125.15"
        ),
    ]
    
    # cuda version lower than tested, missing nvidia_container_toolkit_version
    failure = NvidiaDriverInfo(
        cuda_version_from_nvidia_smi="11.0", driver_version="566.125.15"
    )

    # success case, should not print anything
    success.compare(others)
    printer_echo_mock.assert_not_called()

    # cuda version check should return error as its lower than tested, and container toolkit version is missing
    failure.compare(others)
    printer_echo_mock.assert_has_calls(
        calls=[
            call(
                Printer.minimum_styled(
                    self_value="11.0",
                    supported_values=["12.8", "13.0"],
                    tag="CUDA version from NVIDIA SMI",
                    attr_name="cuda_version_from_nvidia_smi",
                ),
                level="error",
            ),
            call(
                Printer.not_found(
                    tag="NVIDIA Container Toolkit version",
                    attr_name="nvidia_container_toolkit_version",
                ),
                level="error",
            ),
        ],
        any_order=True,
    )


def test_amd_driver_info_compare(printer_echo_mock):
    """
    Similar test as above for AMD
    """
    # all versions in allowed range
    success = AmdDriverInfo(
        cuda_version_from_rocm_smi="12.8", driver_version="566.125.15"
    )
    others = [
        AmdDriverInfo(cuda_version_from_rocm_smi="12.8", driver_version="594.564.56"),
        AmdDriverInfo(cuda_version_from_rocm_smi="13.0", driver_version="566.125.15"),
    ]
    
    # lower cuda version
    failure = AmdDriverInfo(
        cuda_version_from_rocm_smi="11.0", driver_version="566.125.15"
    )

    # no output
    success.compare(others)
    printer_echo_mock.assert_not_called()

    # should print error for cuda version
    failure.compare(others)
    printer_echo_mock.assert_called_once_with(
        Printer.minimum_styled(
            self_value="11.0",
            supported_values=["12.8", "13.0"],
            tag="CUDA version from ROCM SMI",
            attr_name="cuda_version_from_rocm_smi",
        ),
        level="error",
    )


def test_accelerator_info_compare(printer_echo_mock):
    """
    Testing nvidia accelerator info compareagainst other nvidia accelerator infos (apples to apples comparison)
    """
    # version tested
    success = AcceleratorInfo(driver_version="566.125.15", name="NVIDIA B200")
    others = [
        AcceleratorInfo(driver_version="594.564.56", name="NVIDIA B200"),
        AcceleratorInfo(driver_version="566.125.15", name="NVIDIA B200"),
    ]
    # version lower than tested, should generate error
    failure = AcceleratorInfo(driver_version="560.125.15", name="NVIDIA B200")

    success.compare(others)
    printer_echo_mock.assert_not_called()

    failure.compare(others)
    printer_echo_mock.assert_called_once_with(
        Printer.version_compare_styled(
            self_value="560.125.15",
            min_supported_value="566.125.15",
            max_supported_value="594.564.56",
            tag="Driver version",
            attr_name="driver_version",
            greater=False,
        ),
        level="error",
    )


def test_accelerator_compare(printer_echo_mock):
    """
    Test accelerator info object comparison of different types (apples to oranges)
    """
    
    obj = Accelerator.model_validate(
        {
            "nvidia": [
                AcceleratorInfo(driver_version="562.124.012", name="Nvidia H200"),
                AcceleratorInfo(driver_version="562.124.012", name="Nvidia H200"),
            ]
        }
    )
    items = [
        Accelerator.model_validate(
            {
                "nvidia": [
                    AcceleratorInfo(driver_version="562.124.12", name="Nvidia H200"),
                    AcceleratorInfo(driver_version="562.124.12", name="Nvidia H200"),
                ]
            }
        ),
        Accelerator.model_validate(
            {
                "nvidia": [
                    AcceleratorInfo(driver_version="563.124.12", name="Nvidia H200"),
                    AcceleratorInfo(driver_version="561.124.12", name="Nvidia H200"),
                ]
            }
        ),
    ]
    
    # nvidia against nvidia comparison, no errors
    obj.compare(items)
    printer_echo_mock.assert_not_called()

    amd_obj = Accelerator.model_validate(
        {
            "amd": [
                AcceleratorInfo(driver_version="562.124.12", name="AMD Mi355"),
                AcceleratorInfo(driver_version="562.124.12", name="AMD Mi355"),
            ]
        }
    )
    
    # amd against nvidia comparison, should generate error
    amd_obj.compare(items)
    printer_echo_mock.assert_called()
    printer_echo_mock.assert_called_with(
        Printer.list_compare_styled(
            self_value="amd",
            tag="Accelerator",
            supported_values=["nvidia"],
            attr_name="info",
        ),
        level="error",
    )


def test_gpu_info_compare_diff_vendor(printer_echo_mock):
    """ Test GPU info comparison when vendors tested are different, should generate error """
    
    obj = GPUInfo(
        vendor="NVIDIA",
        model="NVIDIA H200",
        ram=64,
        driver_version="576.125.42",
        compute_cap=89,
        index=1,
    )
    amd_items = [
        GPUInfo(vendor="AMD", model="AMD Mi355x", ram=64, driver_version="546.435.56")
    ]
    obj.compare(amd_items)
    printer_echo_mock.assert_called_with(
        "Found no supported vendor configuration for vendor NVIDIA, supported ['AMD']",
        level="error",
    )


def test_gpu_info_compare_pass(printer_echo_mock):
    """
    Test GPU info comparison when vendors are same.
    """
    obj = GPUInfo(
        vendor="NVIDIA",
        model="NVIDIA H200",
        ram=64,
        driver_version="576.125.42",
        compute_cap=89,
        index=1,
    )
    items = [
        GPUInfo(vendor="AMD", model="AMD Mi355x", ram=64, driver_version="546.435.56"),
        GPUInfo(
            vendor="NVIDIA",
            model="NVIDIA H200",
            ram=64,
            driver_version="576.125.42",
            compute_cap=89,
            index=1,
        ),
        GPUInfo(
            vendor="NVIDIA",
            model="NVIDIA H100",
            ram=64,
            driver_version="576.125.42",
            compute_cap=87,
            index=1,
        ),
    ]
    obj.compare(items)
    printer_echo_mock.assert_not_called()
